#pragma implementation "Browse.h"
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <Browse.h>
#include <keyboard.h>
#include "virtpos.h"

TextBrowser::TextBrowser(TextArray &r): txt(r) {
  topline = 0; leftcol = 0;
}

void TextBrowser::move(int r,int c) {
  const char *s;
  if (r < 0 || r >= txt.size())
    r = txt.size();
  while (c < 0 && r > 0 && (s = txt[--r])) {
    virtpos line(s);
    c += line.mapidx(strlen(s)) + 1;
  }
  if (s = txt[r]) {
    virtpos line(s);
    line.mappos(c);
    if (c > leftcol + col) {
      while (c > line.pos && s[line.mappos(++c)])
	;
    }
    if (s[line.idx])
      c = line.pos;
  }
  else
    c = 0;
  int h = loc.height();
  int w = loc.width();
  if (r < topline) {
    int dist = topline - r;
    topline = r;
    slide(Rect(0,0,h,w),DIR_DOWN,dist);
  }
  else if (r >= topline + h) {
    int dist = r - h + 1 - topline;
    topline += dist;
    slide(Rect(0,0,h,w),DIR_UP,dist);
  }
  if (c < leftcol) {
    int dist = leftcol - c;
    leftcol = c;
    slide(Rect(0,0,h,w),DIR_RIGHT,dist);
  }
  else if (c >= leftcol + w) {
    int dist = c - w + 1 - leftcol;
    leftcol += dist;
    slide(Rect(0,0,h,w),DIR_LEFT,dist);
  }
  Browser::curpos(r - topline,c - leftcol);
}

void TextBrowser::draw(const Rect &r) {
  for (int i = r.r1; i <= r.r2; ++i) {
    CHTYPE buf[132];
    virtpos v(txt[i + topline]); v.expand(leftcol + r.c1,buf,r.width());
    write(i,r.c1,r.width(),buf);
  }
}

CHTYPE TextBrowser::operator()(int row, int col) const {
  if (col < 0 || row < 0) return ' ';
  const char *s = txt[row];
  if (!s) return ' ';
  int idx = virtpos(s).mappos(col);
  if (!s[idx]) return ' ';
  return (unsigned char)s[idx];
}

void TextBrowser::getyx(int &y,int &x) const {
  y = row + topline;
  x = col + leftcol;
}

void TextBrowser::print() {
  string cmd("pr -l64 -f -h \"" + getname() + "\" | lp -s");
  FILE *p = popen(cmd.c_str(),"w");
  if (p) {
    for (int i = 0; i < txt.size(); ++i) {
      fputs(txt[i],p);
      fputc('\n',p);
    }
  }
  if (!p || pclose(p)) {
    int rc = errno;
    ErrMsg m(5,10,"Error printing document","%s", sys_errlist[rc]);
    m.run();
  }
  else {
    ErrMsg m(5,10,"Printing Complete","Press Enter");
    m.run();
  }
}
