#include <malloc.h>
#include <bterr.h>
#include "btcb.hpp"
#include "ftype.hpp"

extern "C" {
#include <btflds.h>
  void free(void *);
}

void btFile::close() {
  if (f) ::free(f);
  f = 0;
  klen = 0;
  Btcb::close();
}

int btFile::open(const char *name,int mode) {
  close();
  int rc = Btcb::open(name,mode);
  if (rc == 0 && !(b->flags & BT_DIR)) {
    klen = bt_klen(b->lbuf);
    f = ldflds(0,b->lbuf,b->rlen);
  }
  return rc;
}

btFile::btFile(const char *name,int mode,const Keydata &rec) :
  Btcb(name,mode,rec.size())
{
  rbuf = new Keydata(rec);	// copy rbuf to dynamic memory
  klen = 0;
  f = 0;
  if (b && !(b->flags & BT_DIR)) {
    klen = bt_klen(b->lbuf);
    f = ldflds(0,b->lbuf,b->rlen);
  }
}

btFile::btFile(Keydata rec):
  Btcb(rec.size())
{
  rbuf = new Keydata(rec);
  klen = 0;
  f = 0;
}

btFile::btFile() {
  rbuf = new Keydata;
  klen = 0;
  f = 0;
}

btFile &btFile::operator=(Keydata rec) {
  delete rbuf;
  rbuf = new Keydata(rec);
  setrlen(rec.size());
  return *this;
}

void btFile::unpack(const Keydata *k) {
  char buf[MAXREC];
  b2urec(f->f,buf,k->size(),b->lbuf,b->rlen);
  k->load(buf);
}

void btFile::pack(const Keydata *k) {
  char buf[MAXREC];
  int ksize = k->size();
  u2brec(f->f,buf,k->store(buf),b,ksize < klen ? ksize : klen);
}

void btFile::relate(btFile *r) {
  rel.add(r);
}

int btFile::read(const Keydata &k,enum btop op) {
  pack(&k);	// FIXME: pack only keylength . . .
  b->rlen = rbuf->size();
  int rc = btas(op|NOKEY);
  if (rc == 0) {
    unpack(rbuf);
    for (int i = 0; i < rel.size(); ++i) {
      btFile &r = *rel[i];
      if (r.rbuf->size() > r.klen)
	r.readeq(*r.rbuf);
      else
	r.pack(r.rbuf);
    }
  }
  return rc;
}

int btFile::write() {
  pack(rbuf);
  int rc = btas(BTWRITE|DUPKEY);
  if (rc == 0) {
    for (int i = 0; i < rel.size(); ++i)
      rel[i]->write();
  }
  return rc;
}

int btFile::rewrite() {
  int rc;
  if (!klen) return BTEROPEN;
  char *buf = (char *)alloca(klen);
  b2urec(f->f,buf,klen,b->lbuf,b->rlen);	// current file key
  // check for key/data change
  if (rbuf->compare(buf)) {	// no change
    b->klen = b->rlen;
    btas(BTDELETE|NOKEY);
    pack(rbuf);
    rc = btas(BTWRITE|DUPKEY);
  }
  else {
    pack(rbuf);
    rc = btas(BTREPLACE);
  }
  if (rc == 0) {
    for (int i = 0; i < rel.size(); ++i)
      rel[i]->rewrite();
  }
  return rc;
}

int btFile::del(int op) {
  b->klen = b->rlen;
  int rc = btas(op);
  if (rc == 0) {
    for (int i = 0; i < rel.size(); ++i)
      rel[i]->del();
  }
  return rc;
}

btFile::~btFile() {
  delete rbuf;
  if (f) ::free(f);	// malloced by ldflds
}

#ifdef __GNUC__
template class Array<btFile *>;
#endif

int btFile::read(enum btop op) {
  Keydata k(*rbuf,klen);
  return read(k,op);
}
