#pragma implementation
#include <stdio.h>
#include <fcntl.h>
#include <signal.h>
#include <keyboard.h>
#include "cmd.h"

CmdBrowser::CmdBrowser(string s): PipeBrowser(s.c_str()) {
  p = popen(s.c_str(),"r");
  if (p)
    open(fileno(p));
}

CmdBrowser::~CmdBrowser() {
  if (p)
    pclose(p);
}

void PipeBrowser::handler() {
  pipe.reseteof();
  long newsize = pipe.size();
  /* fprintf(stderr,"new=%ld,old=%ld,tail=%d\r\n",newsize,oldsize,tail); */
  if (newsize == oldsize && pipe.ateof()) {
    disarm();
    return;
  }
  if (!tail) {
    if (oldsize < topline + loc.height())
      redraw();
  }
  else if (newsize > oldsize) {
    if (ishidden()) top();
    move(newsize,0);
    redraw();
    sync();
    Ksync();
  }
  oldsize = newsize;
}

void PipeBrowser::key(int k) {
  switch (k) {
  case 'G':
    long newsize = pipe.size();
#if 0
    if (oldsize < newsize) {
      static const char *txt[] = { "Press any key to interrupt", 0 };
      TextWindow msg(txt,"Searching for EOF");
      msg.setpos(Point(10,20));
      Ksync();
      do {
	/* move(newsize,0); */
	/* redraw(); */
	/* PSsync(); */
	oldsize = newsize;
	newsize = pipe.size();
      } while (!Kready() && oldsize < newsize);
    }
#endif
    move(oldsize = newsize,0);
    redraw();
    sync();
    tail = ~tail;
    return;
  }
  /* fprintf(stderr,"key=%d\n\r",k); */
  TextBrowser::key(k);
}

PipeBrowser::PipeBrowser(const char *s):
  TextBrowser(pipe), SelectPoll(&SelectRead)
{
  name = s;
  oldsize = 0;
  tail = 0;
  frame->set(name.c_str());
}

void PipeBrowser::open(int f) {
  fcntl(f,F_SETFL,O_NDELAY);
  pipe.open(f);
  oldsize = 0;
  arm(f);
}

PipeBrowser::PipeBrowser(int f,const char *s):
  TextBrowser(pipe), SelectPoll(&SelectRead)
{
  name = s;
  tail = 0;
  frame->set(name.c_str());
  open(f);
}

PipeBrowser::~PipeBrowser() { }
