#ifdef __GNUC__
#pragma implementation "Curses.h"
#endif
#include "Curses.h"

CursesWin::CursesWin(int nrow,int ncol) {
  d = 0;
  data = 0;
  attr = A_NORMAL;
  scrollflag = 0;
  crow = ccol = 0;
  size(nrow,ncol);
}

void CursesWin::size(int nrow,int ncol) {
  delete [] d;
  delete [] data;
  rows = nrow;
  cols = ncol;
  int n = rows * cols;
  data = new CHTYPE far [n];
  int i;
  for (i = 0; i < n; ++i)
    data[i] = ' ' + A_NORMAL;
  d = new rowdata [rows];
  for (i = 0; i < rows; ++i) {
    d[i].modc = 0;
    d[i].data = cols * i;
  }
  wrow = wcol = 0;
  if (crow >= rows)
    crow = 0;
  if (ccol >= cols)
    ccol = 0;
}

CursesWin::~CursesWin() {
  delete [] d;
  delete [] data;
}

void CursesWin::draw(const Rect &rs) {
  Rect vloc(-wrow,-wcol,rows-wrow-1,cols-wcol-1);
  Rect r(rs.intersect(vloc));
  int w = r.width();
  int col = r.c1 + wcol;
  for (int i = r.r1; i <= r.r2; ++i)
    write(i,r.c1,w,data + d[i + wrow].data + col);
  if (r.c2 < rs.c2)
    fill(rs.intersect(Rect(0,w)),'%');
  if (r.c1 > rs.c1)
    fill(rs.intersect(Rect(0,0,255,r.c1-1)),'%');
  if (r.r2 < rs.r2)
    fill(rs.intersect(Rect(r.height(),0)),'^');
  if (r.r1 > rs.r1)
    fill(rs.intersect(Rect(0,0,r.r1-1)),'^');
}

#if 0
void CursesWin::setphys(const Rect &r) {
  if (r.width() > cols)
    setphys(Rect(r.r1,r.c1,r.r2,r.c1 + cols - 1));
  else if (r.height() > rows)
    Window::setphys(Rect(r.r1,r.c1,r.r1 + rows - 1,r.c1));
  else
    Window::setphys(r);
}
#endif

void CursesWin::curpos(int row,int col) {
  bool moved = 0;
  if (row < wrow)
    wrow = row, moved = 1;
  if (col < wcol)
    wcol = col, moved = 1;
  int w = loc.height();
  if (row >= wrow + w)
    wrow = row - w + 1, moved = 1;
  w = loc.width();
  if (col >= wcol + w)
    wcol = col - w + 1, moved = 1;
  if (moved)
    redraw();
  Window::curpos(crow - wrow,ccol - wcol);
}

void CursesWin::sync() {
  curpos(crow,ccol);
  Rect vloc(-wrow,-wcol,rows-wrow-1,cols-wcol-1);
  Rect r(vloc.moverel(loc.r1,loc.c1).intersect(loc));
  int h = r.height();
  int w = r.width();
  for (int i = 0; i < h; ++i) {
    rowdata *p = d + i + wrow;
    if (p->modc < w + wcol)
      write(i,p->modc - wcol,cols - p->modc,data + p->data + p->modc);
    p->modc = cols;
  }
  //Window::sync();
}

void CursesWin::addch(char c) {
  switch (c) {
  case '\n':
    ccol = cols;
    break;
  case '\t':
    ccol += 8 - ccol % 8;
    break;
  default:
    struct rowdata *p = d + crow;
    CHTYPE ch = (unsigned char)c | attr;
    if (ch != data[p->data+ccol]) {	/* if char or attr changed */
      if (ccol < p->modc)
	p->modc = ccol;		/* update first change column */
      data[p->data + ccol] = ch;
    }
    ++ccol;
  }
  if (ccol >= cols) {
    if (ccol = 0,++crow >= rows)
      if (!scrollok())
	crow = 0;
      else {
	--crow;
	deleteln(0);
      }
  }
}

void CursesWin::addstr(const char *s) {
  while (*s) {
    addch(*s++);
    if (ccol + crow == 0) break;
  }
}

void CursesWin::insch(CHTYPE ch) {
  CHTYPE far *p = data + d[crow].data;
  if (d[crow].modc > ccol) d[crow].modc = ccol;
  for (int i = cols; --i > ccol; p[i] = p[i - 1]);
  p[ccol] = ch;
}

void CursesWin::delch() {
  CHTYPE far *cp = data + d[crow].data;
  int i;
  for (i = ccol; ++i < cols; cp[i-1] = cp[i]);
  cp[--i] = ' ' | attr;
  if (d[crow].modc > ccol) d[crow].modc = ccol;
}

void CursesWin::deleteln(int r) {
  rowdata p = d[r];
  int i;
  for (i = r + 1; i < rows; ++i) {
    d[i].modc = 0;
    d[i-1] = d[i];
  }
  p.modc = 0;
  d[--i] = p;
  CHTYPE far *cp = data + p.data;
  CHTYPE ch = ' ' | attr;
  for (i = 0; i < cols; ++i)
    cp[i] = ch;
}

void CursesWin::insertln(int r) {
  int i = rows - 1;
  rowdata p = d[i];
  while (--i >= r) {
    d[i].modc = 0;
    d[i+1] = d[i];
  }
  p.modc = 0;
  d[r] = p;
  CHTYPE far *cp = data + p.data;
  CHTYPE ch = ' ' | attr;
  for (i = 0; i < cols; ++i)
    cp[i] = ch;
}

void CursesWin::clrtoeos(CHTYPE ch) {
  clrtoeol(ch);
  for (int r = crow + 1; r < rows; ++r) {
    CHTYPE far *cp = data + d[r].data;
    for (int i = 0; i < cols; ++i)
      cp[i] = ch;
    d[r].modc = 0;
  }
}

void CursesWin::clrtoeol(CHTYPE ch) {
  CHTYPE far *cp = data + d[crow].data;
  for (int i = ccol; i < cols; ++i)
    cp[i] = ch;
  if (d[crow].modc > ccol) d[crow].modc = ccol;
}

void CursesWin::move(int row,int col) {
  if (row < 0) row += rows;
  if (col < 0) col += cols;
  crow = (unsigned)row % rows;
  ccol = (unsigned)col % cols;
}

CHTYPE &CursesWin::operator()(int row,int col) {
  rowdata *p = d + row;
  if (col < p->modc)
    p->modc = col;
  return data[p->data + col];
}
