#pragma implementation "EditWin.h"
#include <stdio.h>
#include <ctype.h>
#include <keyboard.h>
#include <EditWin.h>
#include <Prompt.h>

Browser::Browser() { }
Browser::~Browser() { }

void Browser::setphys(const Rect &r1) {
  int r,c;
  getyx(r,c);
  MainWin::setphys(r1);
  move(r,c);
}

EditWin::EditWin() { linemode = 0; }
EditWin::~EditWin() { }

void Browser::cmd(int c) {
  int currow,curcol;
  getyx(currow,curcol);
  int topline = currow - row;
  switch (c) {
  case EDIT_SEARCH:
    { PromptWin p("Search String:",pat);
      if (p.run() == KEY_F(1)) return;
      pat = p.get();
    }
    // fall through
  case EDIT_AGAIN:
    search(pat.c_str());
    return;
  case EDIT_FWD:
    move(topline + 2 * loc.height() - 1,curcol);
    move(currow + loc.height(),curcol);
    return;
  case EDIT_REV:
    if (currow <= loc.height())
      move(0,curcol);
    else {
      move(topline - loc.height(),curcol);
      move(currow - loc.height(),curcol);
    }
    return;
  case FILE_PRINT:
    print();
    return;
  }
  MainWin::cmd(c);
}

void EditWin::cmd(int c) {
  int currow,curcol;
  switch (c) {
  case EDIT_INSLINE:
    getyx(currow,curcol);
    insertln(currow);
    sync();
    return;
  case EDIT_DELLINE:
    getyx(currow,curcol);
    deleteln(currow);
    sync();
    return;
  }
  Browser::cmd(c);
}

void Browser::key(int ky) {
  int currow, curcol;
  int inc;
  getyx(currow,curcol);
  switch (ky) {
  case KEY_F(5):
    cmd(EDIT_REFORM);
    break;
  case KEY_FIND: case KEY_F(11): case '/':
    cmd(EDIT_SEARCH);
    break;
  case KEY_SFIND: case KEY_F(16): case 'n':
    cmd(EDIT_AGAIN);
    break;
  case KEY_SHOME: case 'T':
    move(0,0);
    break;
  case KEY_UP: case 'k':
    move(DIR_UP);
    break;
  case KEY_DOWN: case 'j':
    move(DIR_DOWN);
    break;
  case KEY_LEFT: case KEY_BACKSPACE: case 'h':
    move(DIR_LEFT);
    break;
  case KEY_RIGHT: case 'l':
    move(DIR_RIGHT);
    break;
  case KEY_ENTER: case KEY_LL:
    move(++currow,0);
    break;
  case KEY_F(2): case KEY_PPAGE:	// prev PAGE
  case KEY_SR: case 2:			// prev SCREEN
    cmd(EDIT_REV);
    break;
  case KEY_F(3): case KEY_NPAGE:	// next PAGE
  case KEY_SF: case 6:			// next SCREEN
    cmd(EDIT_FWD);
    break;
  case 'H':	// top of window
    move(currow - row,curcol);
    break;
  case 'M':	// middle of window
    move(currow - row + loc.height() / 2,curcol);
    break;
  case 'L':	// bottom of window
    move(currow - row + loc.height() - 1,curcol);
    break;
  case '<':		// 52 columns left
    inc = 52;
    if (curcol <= inc)
      move(currow,0);
    else {
      move(currow, curcol - col - inc);
      move(currow,curcol - inc);
    }
    break;
  case '>':		// 52 columns right
    inc = 52;
    move(currow,curcol - col + loc.width() - 1 + inc);
    move(currow,curcol + inc);
    break;
  case KEY_SLEFT:	// shift screen left
    if (curcol > 0) {
      inc = col;
      move(currow,curcol - col - 1);
      if (inc >= loc.width() - 1)
	--curcol;
      move(currow,curcol);
    }
    break;
  case KEY_SRIGHT:	// shift screen right
    inc = col;
    move(currow,curcol - col + loc.width());
    if (inc == 0)
      ++curcol;
    move(currow,curcol);
    break;
  case 4:	// half page forward (^D)
    inc = loc.height() / 2;
    move(currow - row + loc.height() - 1 + inc,curcol);
    move(currow + inc,curcol);
    break;
  case 5:	// one line forward (^E)
    inc = row;
    move(currow - row + loc.height(),curcol);
    if (inc == 0)
      ++currow;
    move(currow,curcol);
    break;
  case 21:	// half page back (^U)
    inc = loc.height() / 2;
    if (currow <= inc)
      move(0,curcol);
    else {
      move(currow - row - inc,curcol);
      move(currow - inc,curcol);
    }
    break;
  case 25:	// one line back (^Y)
    inc = row;
    move(currow - row - 1,curcol);
    if (inc >= loc.height() - 1)
      --currow;
    move(currow,curcol);
    break;
  case KEY_END: case '$':
    move(currow+1,-1);
    break;
  case KEY_HOME: case '0':
    move(currow,0);
    break;
  case KEY_TAB:
    move(currow,(curcol & ~7) + 8);
    break;
  case KEY_BTAB:
    if (curcol > 7)
      move(currow,(curcol & ~7) - 8);
    break;
  case KEY_SEND: case 'G':
    move(-1,-1);
    break;
  default:
    MainWin::key(ky);
    return;
  }
  sync();
}

// FIXME: should maintain in insert_mode, not linemode
enum { CMD_MODE = 0x40 };

void EditWin::key(int ky) {
  int currow, curcol;
  if (linemode & CMD_MODE) {
    switch (ky) {
    case 'x':
      delch();
      sync();
      return;
    case 's':
      delch();
      break;
    case 'a':
      Browser::key(KEY_RIGHT);
    case 'i':
      break;
    case 'o':
      Browser::key(KEY_DOWN);
    case 'O':
      cmd(EDIT_INSLINE);
      break;
    case 'R':
      insert_mode = 0;
      linemode &= ~CMD_MODE;
      sync();
      return;
    default:
      Browser::key(ky);
      return;
    }
    insert_mode = 1;
    linemode &= ~CMD_MODE;
    sync();
    return;
  }
  switch (ky) {
  case 27:
    linemode |= CMD_MODE;
    break;
  case KEY_F(7): case KEY_IL:
    cmd(EDIT_INSLINE);
    return;
  case KEY_F(8): case KEY_DL:
    cmd(EDIT_DELLINE);
    return;
  case KEY_SELECT:
    if (linemode = ++linemode % 3)
      last_dir = DIR_NONE;
    break;
  case KEY_IC:
    insert_mode = ~insert_mode;
    break;
  case KEY_UP:
    if (linemode) linemove(DIR_UP);
    move(DIR_UP);
    break;
  case KEY_DOWN:
    if (linemode) linemove(DIR_DOWN);
    move(DIR_DOWN);
    break;
  case KEY_LEFT:
    if (linemode) linemove(DIR_LEFT);
    move(DIR_LEFT);
    break;
  case KEY_RIGHT:
    if (linemode) linemove(DIR_RIGHT);
    move(DIR_RIGHT);
    break;
  case KEY_BACKSPACE:
    getyx(currow,curcol);
    if (linemode) {
      linedel();
      break;
    }
    if (curcol > 0) {
      --curcol;
      move(currow,curcol);
      delch();
    }
    break;
  case KEY_DC:
    delch();
    break;
  case KEY_EOL:
    clrtoeol();
    break;
  default:
    if (isascii(ky) && isprint(ky)) {
      if (insert_mode)
	insch(ky);
      else
	addch(ky);
      break;
    }
    Browser::key(ky);
    return;
  }
  sync();
}

void Browser::sync() {
  int row, col;
  getyx(row,col);
  sprintf(desc,"Row: %2d Col: %2d", row, col);
  const string &n = getname();
  frame->set(n.c_str(),desc);
}

void EditWin::sync() {
  int row, col;
  getyx(row,col);
#if 0
  ostrstream os(desc,sizeof desc);
  os << "Row:" << row << " Col:" << col;
  if (modified)
    os << " (modified)";
  os << ends;
  os.freeze();
#else
  char lmode[16];
  lmode[0] = 0;
  if (linemode & ~CMD_MODE)
    sprintf(lmode,"(line%d)",linemode & ~CMD_MODE);
  sprintf(desc,"Row: %2d Col: %2d%s%s%s%s", row, col,
	modified ? " (mod)" : "",
	insert_mode ? " (ins)" : "",
	(linemode & CMD_MODE) ? " (cmd)" : "",
	lmode
  );
#endif
  const string &n = getname();
  frame->set(n.length() ? n.c_str() : "(untitled)",desc);
}

void EditWin::clrtoeol() {
  // don't know how to do this generically
}

void Browser::move(Dir d) {
  int currow, curcol;
  getyx(currow,curcol);
  switch (d) {
  case DIR_UP:
    if (currow) move(--currow,curcol);
    break;
  case DIR_DOWN:
    move(++currow,curcol);
    break;
  case DIR_LEFT:
    move(currow,--curcol);
    break;
  case DIR_RIGHT:
    move(currow,++curcol);
    break;
  }
}

Menu &EditWin::editMenu() {
  return *new PopupMenu("Edit Menu")
    + new CmdMenuItem("InsLine (F7)",EDIT_INSLINE)
    + new CmdMenuItem("DelLine (F8)",EDIT_DELLINE)
    + new CmdMenuItem("MarkBlk (F9)",EDIT_MARKBLK)
    + new CmdMenuItem("Paste   (F6)",EDIT_MARKBLK)
    + new CmdMenuItem("Refresh (F5)",EDIT_REFORM)
    + new CmdMenuItem("Graphics",EDIT_GRAPH)
    + new CmdMenuItem("Wordwrap(F14)",EDIT_WORDWRAP)
    + new CmdMenuItem("Search  (F11)",EDIT_SEARCH)
    + new CmdMenuItem("Again   (F16)",EDIT_AGAIN)
  ;
}
