#ifdef __GNUC__
#pragma implementation "Fileio.h"
#endif
#include <string.h>
#include "Fileio.h"

#if 0
int FileIO::isnull() const {
  char *p = nullbuf;
  const keypart *kp;
  int i;
  for (i = 0, kp = k->k_part; i < k->k_nparts; ++i,++kp) {
    if (memcmp(p,data+kp->kp_start,kp->kp_leng)) return 0;
    p += kp->kp_leng;
  }
  return 1;	/* return non-zero if current record has "null" key */
}

void FileIO::setnull() {	/* set the "null" key used by isnull() */
  int i;
  char *p = nullbuf;
  const struct keypart *kp;
  for (i = 0, kp = k->k_part; i < k->k_nparts; ++i,++kp) {
    memcpy(p,data + kp->kp_start,kp->kp_leng);
    p += kp->kp_leng;
  }
}
#endif

void FileIO::blank() {
  memset(data,0,len);
}

char *FileIO::tag() {	/* return tag of current record */
  char *p = tagbuf;
  int i;
  register const struct keypart *kp;
  for (i = 0, kp = k->k_part; i < k->k_nparts; ++i,++kp) {
    memcpy(p,data + kp->kp_start,kp->kp_leng);
    p += kp->kp_leng;
  }
  return tagbuf;
}

void FileIO::tag(Tag p) {
  if (!p) return;
  int i;
  register const struct keypart *kp;
  for (i = 0, kp = k->k_part; i < k->k_nparts; ++i,++kp) {
    memcpy(data + kp->kp_start,p,kp->kp_leng);
    p += kp->kp_leng;
  }
}

/* should call errpost() on errors to handle lazy programmers */

FileIO::FileIO(int f,void *rbuf) {
  int i;
  fd = f;
  data = (char *)rbuf;
  struct dictinfo d;
  if (isindexinfo(fd,(struct keydesc *)&d,0) == 0)
    len = d.di_recsize;
  else
    len = 0;
  k = new keydesc;
  k->k_nparts = 0;
  isindexinfo(fd,k,1);
  tagl = 0;
  for (i = 0; i < k->k_nparts; ++i)
    tagl += k->k_part[i].kp_leng;	/* calculate key size */
  if (tagl == 0) tagl = 4;		// isrecnum key
  tagbuf = new char[tagl * 2];
  nullbuf = tagbuf + tagl;
  min = max = 0;
  vw = k;
  curr_vw = k;
  dir = 0;
}

FileIO::~FileIO() {
  delete k;
  delete [] tagbuf;
}

FcbIO::Tag FileIO::get(Tag tg) {	// get for update (lock)
  tag(tg);
  return isread(fd,data,ISEQUAL) ? 0 : tag();
}

FcbIO::Tag FileIO::update(Tag) {
  int rc = isrewcurr(fd,data);
  if (rc == ENOCURR)
    rc = iswrite(fd,data);
  return rc ? 0 : tag();
}

FcbIO::Tag FileIO::add(Tag,int where) {
  if (where)
    /* should reserve hidden key here.  When there is no hidden
       key, returning null causes FCBADD_AFTER to be called later
       to append the record. */
    return 0;
  return iswrite(fd,data) ? 0 : tag();
}

FcbIO::Tag FileIO::locate(Tag tg) {
  tag(tg);
  return (isread(fd,data,ISGTEQ) && isread(fd,data,ISLAST)) ? 0 : tag();
}

FcbIO::Tag FileIO::next(int cnt) {
  while (cnt > 1) {
    if (next() == 0) return 0;
    --cnt;
  }
  while (cnt < -1) {
    if (prev() == 0) return 0;
    ++cnt;
  }
  if (cnt == 1) {
    if (isread(fd,data,ISNEXT)) return 0;
  }
  if (cnt == -1) {
    if (isread(fd,data,ISPREV)) return 0;
  }
  return tag();
}

FcbIO::Tag FileIO::first(int cnt) {
  if (cnt >= 0) {
    if (isread(fd,data,ISFIRST)) return 0;
    while (cnt--)
      if (isread(fd,data,ISNEXT)) return 0;
  }
  else {
    if (isread(fd,data,ISLAST)) return 0;
    while (++cnt < 0)
      if (isread(fd,data,ISPREV)) return 0;
  }
  return tag();
}

FcbIO::Tag FileIO::del(Tag tg) {
  tag(tg);
  return isdelete(fd,data) ? 0 : tg;
}

void FileIO::clear() {}
