#pragma implementation "MainWin.h"
#include <Array.h>
#include <MainWin.h>
#include <ListBox.h>
#include <keyboard.h>

template class Array<MainWin *>;

MDIFrame::~MDIFrame() {
  for (int i = 0; i < win.size(); ++i)
    delete win[i];
}

MDIFrame::MDIFrame() {
  rows = Window::screen.height() - 1;
  if (rows < 24)
    rows = 24;
  cur = 0;
  active = 0;
  menu = new BarMenu;
  *menu + new SubMenu("File",*new PopupMenu("File Menu")
      + new CmdMenuItem("New (F7)",FILE_NEW)
      + new CmdMenuItem("Open",FILE_OPEN)
      + new CmdMenuItem("Save",FILE_SAVE)
      + new CmdMenuItem("Save~As",FILE_SAVEAS)
      + new CmdMenuItem("Print", MainWin::FILE_PRINT)
      + new CmdMenuItem("E~xit",EXIT)
    )
  + new SubMenu("Window",*new PopupMenu("Window Menu")
      + new CmdMenuItem("Next (F13)",WINDOW_NEXT)
      + new CmdMenuItem("Prev (F12)",WINDOW_PREV)
      + new CmdMenuItem("Size (F15)",WINDOW_SIZE)
      + new CmdMenuItem("Zoom (F17)",WINDOW_ZOOM)
      + new CmdMenuItem("Close (F1)",WINDOW_CLOSE)
      + new CmdMenuItem("List (F20)",WINDOW_LIST)
    )
  ;
}

void MainWin::key(int ky) {
  switch (ky) {
  case KEY_F(6):
    cmd(EDIT_PASTE);
    return;
  case KEY_F(9):
    cmd(EDIT_MARKBLK);
    return;
  }
  Karet::key(ky);
}

void MainWin::getfocus() {
  frame->settype(1);
  top();
}

void MainWin::losefocus() {
  frame->settype(0);
}

void MDIFrame::key(int ky) {
  switch (ky) {
  case KEY_F(1):
    if (!win.size()) break;
    cmd(WINDOW_CLOSE);
    if (!win.size() && menu)
      menu->popupAt(Point(0,0),this);
    return;
  case KEY_F(20):
    cmd(WINDOW_LIST);
    return;
  case KEY_F(17):
    cmd(WINDOW_ZOOM);
    return;
  case KEY_F(7):
    cmd(FILE_NEW);
    return;
  case KEY_F(15):
    cmd(WINDOW_SIZE);
    return;
  case KEY_F(12):
    cmd(WINDOW_PREV);
    return;
  case KEY_F(13):
    cmd(WINDOW_NEXT);
    return;
  }
  bmsApp::key(ky);
}

void MDIFrame::addwin(MainWin *w) {
  for (cur = 0; cur < win.size(); ++cur) {
    if (win[cur] == w) {
      win.del(cur);
      break;
    }
  }
  cur = win.size();
  win.add(w);
  setfocus(active = w);
}

void MDIFrame::open(const string &name) {
  if (active = fileNew()) {
    addwin(active);
    active->load(name);
  }
}

void MDIFrame::cmd(int c) {
  switch (c) {
  case FILE_NEW:
    if (active = fileNew())
      addwin(active);
    return;
  case FILE_SAVE:
    if (active) {
      active->save();
      setfocus(active);
    }
    return;
  case FILE_SAVEAS:
    if (active) {
      active->saveAs("");
      setfocus(active);
    }
    return;
  case WINDOW_NEXT:
    if (++cur >= win.size())
      cur = 0;
    if (cur < win.size())
      setfocus(active = win[cur]);
    else
      active = 0;
    return;
  case WINDOW_CLOSE:
    if (!active || !active->queryOk()) return;
    delwin(active);
    return;
  case WINDOW_PREV:
    if (win.size() && --cur < 0)
      cur = win.size() - 1;
    if (cur < win.size())
      setfocus(active = win[cur]);
    else
      active = 0;
    return;
  case WINDOW_ZOOM:
    if (active) {
      if (active->isZoomed())
	active->unzoom();
      else {
	active->zoom(Rect(0,0,rows - 1,Window::screen.width()-1));
      }
      setfocus(active);
    }
    return;
  case WINDOW_SIZE:
    if (active) {
      active->sizeFromUser();
      setfocus(active);
    }
    return;
  case WINDOW_LIST:
    { MainWin *w = pickList();
      if (w)
	setfocus(active = w);
    }
    return;
  // these are backward - should be cmd() in MainWin!
  case EXIT:
    closeAll();
    if (win.size()) return;	// cancel EXIT if not all closed
  }
  if (active) {
    setfocus(active);
    active->cmd(c);
  }
  else
    bmsApp::cmd(c);
}

bool MDIFrame::closeAll() {
  active = 0;
  for (int i = 0; i < win.size(); ++i) {
    if (win[i]->queryOk() == 0) return FALSE;
    delwin(win[i--]);
  }
  return TRUE;
}

void MDIFrame::delwin(ChildWin *w) {
  int i;
  for (i = 0; i < win.size(); ++i) {
    if (win[i] == w) {
      win.del(i);
      break;
    }
  }
  delete w;
  if (active && i <= cur)
    cmd(WINDOW_PREV);
}

MainWin::MainWin(): FramedWin("(Untitled)") {
  modified = 0;
}

MainWin::~MainWin() { }

bool MainWin::queryOk() {
  if (modified) {
    string s(getname() + " is modified!");
    PopupMenu m(s.c_str());
    m + new MenuItem("Save Changes")
      + new MenuItem("Discard Changes")
      + new MenuItem("Cancel")
    ;
    setfocus(this);
    switch (m.popupAt(Point(10,5))) {
    case 0:
      save();
      if (!modified) break;
    case 2:
      return false;
    }
  }
  return true;
}

void MainWin::save() {
  saveAs(name);
}

MainWin *MDIFrame::pickList() {
  if (win.size() == 0) return 0;
  StringArray list;
  for (int j = 0; j < win.size(); ++j)
    list.put(win[j]->getTitle());
  /* list.sort(); */
  bmsListBox listbox("Window List",list);
  Rect &r = Window::gettop()->loc;
  listbox.setpos(Point(r.r1,r.c1));
  setfocus(&listbox);
  if (listbox.run() == KEY_F(1)) return 0;
  int i = listbox.get();
  return (i < win.size()) ? win[i] : 0;
}

void MDIFrame::position(Window *w,int lines,int cols) {
  Rect big(Window::screen.expand(-1));
  if (active) {
    if (big.r2 - active->loc.r2 >= lines + 2)
      w->setphys(big.intersect(
	Rect(Point(active->loc.r2 + 3,active->loc.c1),cols,lines)
      ));
    else
      w->setphys(big.intersect(
	Rect(active->loc.origin() + Point(1,1),cols,lines)
      ));
  }
  else
    w->setphys(big.intersect(Rect(1,1,lines,cols)));
}
