#include "Money.h"
#include <string.h>
#include <stdlib.h>

string Money::pic(const char *mask) const {
  string s(mask);
  pic(mask,(char *)s.data());
  return s;
}

enum { DECIMAL, COMMA, LBRACK, RBRACK, CURR };

char *Money::pic(const char *mask,char *outptr)	const {
  static const char *spec;
  char	fill = ' ',		/* fill character */
	sign = '+';
  int masklen;			/* length of mask in bytes */
  register char *p;		/* current buffer position */
  register int c;		/* current character */
  char *start, *startd;		/* last & first digit pointers */
  Money num(*this);		// copy of number

  if (!spec) if (!(spec=getenv("PIC"))) spec = ".,<>$";

  for (p=outptr; *mask ; *p++ = *mask++);	/* copy mask to buffer */
  masklen = p-outptr;				/* save mask length */

  if (sgn()<0) {				/* find sign of number */
    sign = '-';
    num.mul(-1);				// take absolute value
  }

  /* find last digit, blank if positive, p -> end of mask field */

  while (--p>=outptr && (c = *p)!='#' && c!='Z' )
    if (sign == '+') {				/* if positive */
      if (c!='+' && c!='%') *p = ' ';		/* blank all but +,% */
    }
    else {
      if (c=='+') *p = '-';	/* all but + unchanged if negative */
      if (c=='>') *p = spec[RBRACK];
    }

/* replace Z & # with digits, Z's replaced with blank instead of a leading
   zero except after (before since we are going backwards) the decimal */

  startd = start = ++p;
  while (p>outptr) {
    c = *--p;
    if (c=='.') {
      *p=spec[DECIMAL];
      if (startd!=start) {
	register char *r1=startd;
	while (r1!=p) {
	  if ( *r1=='Z') *r1='0'; /* replace Z with 0 after decimal */
	  --r1;
	}
	startd = p;
      }
    }
    if (c==',') *p = spec[COMMA];
    if ( c=='#' || (c=='Z' && num.sgn()) ) {
      *p = num.div(10)+'0';
      startd = p;
    }
  }

  if (num.sgn()) memset(outptr,'*',masklen);	/* check overflow */

  if ((fill = *p)!='*') fill = ' ';	/* determine fill character */

/* insert floating character before first digit */

  {
    register int floatc = fill;
    while (p<startd) {
      if ((c = *p)=='$') floatc = spec[CURR];
      if (c=='+' || c==sign) floatc = sign;
      if (c=='<' && sign == '-') floatc = spec[LBRACK];
      *p++ = fill;
    }
    if (p>outptr && sgn()) *--p = floatc;	/* insert floatc char */
  }
  return outptr;
}
