#pragma implementation "scrScreen.h"
//#include <iostream.h>
#include <unistd.h>
#include <memory.h>
#include <fcntl.h>
#include "scrScreen.h"

extern "C" int pathopen(const char *,int);

struct fldtbl {
  char row, col, len, min, lcnt;
  char nul;
  fldtbl() { memset(this,0,sizeof *this); }
};

scrScreen::scrScreen(short r,short c): CursesWin(r,c) {
  int maxrows = rows;
  if (maxrows > 24) maxrows = 24;
  flds = new fldtbl[maxrows*cols/2 + 1];
  flds->lcnt = 1;	/* initialize field -1 */
  ++flds;
  nflds = 0;
}

scrScreen::~scrScreen() {
  if (flds)
    delete [] (flds - 1);
}

void scrScreen::addline(short row,CHTYPE *buf,short len) {
  CHTYPE *ptr = buf;
  fldtbl *g, *f = flds + nflds;
  short col;	/* current column/field length */
  short rem = len;	/* bytes remaining */
  CHTYPE c;

/* extract fields replacing with blanks */

  for (;;) {
    for (col = 0; col < rem; ++col,++ptr) {
      c = (*ptr & A_CHARTEXT);
      if (c == '_' || c == '~') break;	/* search for null character */
    }
    if (col >= rem) break;

    /* compute fld table entry */

    f->row = row; f->col = len - rem + col;
    c = *ptr;
    f->nul = c;
    for (rem -= col,col = 0; col < rem && *ptr == c; ++col)
      *ptr++ = (c & A_ATTRIBUTES) | ' ';
    rem -= col;
    f->len = col;

    /* install in flds */

    for (g=f;;) {
      if (--g<flds) {
	f->lcnt = 1;
	(f++)->min = row;	/* new field */
	++nflds;
	break;
      }
      if (g->col==f->col && g->len==col &&
	  g->min+g->lcnt==row &&
	  (g->nul & A_CHARTEXT) == (f->nul & A_CHARTEXT)) {
	++g->lcnt;		/* new line */
	break;
      }
    }
  }

  move(row,0);
  while (len--)
    addch(*buf++);	/* display line */
}

int scrScreen::dispscrn(const char *name) {
  int fd = pathopen(name,O_RDONLY);
  if (fd < 0) return -1;

  /* display file fixed at 24 X 80 for portability for now */

  unsigned char scrbuf[4096], *p = scrbuf;
  int i,j;
  i = read(fd,(char *)scrbuf,sizeof scrbuf);
  if (i < 1920) return -1;
  (void)close(fd);

  int row = 0;
  if (rows > 24)
    row = (rows - 24) >> 1;	/* center in long display */
  int maxrows = rows;
  if (maxrows > 24) maxrows = 24;

  (void)memset((char *)scrbuf+i,0,sizeof scrbuf - i);
  for (i = 0; i < maxrows; ++i) {
    CHTYPE linebuff[80];
    for (j = 0; j < 80; ++p)
      linebuff[j++] = (p[0] | (p[1920] << 8));
    addline(row++,linebuff,cols);
  }
  sync();		/* let operator read cereal box */
  return 0;
}

Rect scrScreen::fldpos(short i) const {
  const fldtbl *f = &flds[i];
  if (i < 0 || i >= nflds)
    return Rect(1,1,0,0);
  return Rect(f->row,f->col,f->row + f->lcnt - 1,f->col + f->len - 1);
}
