#ifdef __GNUC__
#pragma implementation "Screen.h"
#endif
//#include <iostream.h>
#include <ctype.h>
#include <keyboard.h>
#include "Screen.h"

bmsField::bmsField(ChildWin *p,short l,short r): ChildWin(p), CursesWin(r,l) {}

void bmsField::mark(CHTYPE a) {
  int row,col;
  getyx(row,col);
  move(0,0);
  attr = a;
  for (int i = 0; i < rows; ++i)
    for (int j = 0; j < cols; ++j)
      addch(inch() & A_CHARTEXT);
  move(row,col);
}

void bmsField::losefocus() {
  mark(A_NORMAL);
  sync();
}

void bmsField::put(char c) {
  //cerr << "put(" << c << ")\n";
  if (insert_mode)
    insch(c);
  addch(c);
}

void bmsField::key(int k) {
  switch (k) {
  case KEY_EOL:
    clrtoeol();
    break;
  case KEY_END:
    ccol = get().length();
    break;
  case KEY_LEFT:
    if (ccol <= 0) break;
    --ccol;
    break;
  case KEY_RIGHT:
    if (ccol >= cols - 1) break;
    ++ccol;
    break;
  case KEY_BACKSPACE:
    if (ccol > 0) {
      --ccol;
      if (insert_mode)
	delch();
      else
	put(' '),--ccol;
      sync();
    }
    return;
  case KEY_DC:
    delch();
    break;
  default:
    if (isascii(k) && isprint(k)) {
      put(k);
      sync();
      return;
    }
    Karet::key(k);
    return;
  }
  sync();
  return;
}

string bmsField::get() const {
  string str;
  str.reserve(rows * cols);
  int pos = 0;
  // don't do rows yet
  for (int i = 0; i < rows; ++i) {
    for (int j = 0; j < cols; ++j)
      str[pos++] = (*this)(i,j);
    while (pos && str[pos-1] == ' ') --pos;
    if (rows > 1) str[pos++] = '\n';
  }
  str.erase(pos,str.length() - pos);
  return str;
}

bmsField &bmsField::operator=(string r) {
  move(0,0);
  int size = rows * cols;
  addstr(r.c_str());
  if (r.length() < size)
    clrtoeos();
  sync();
  return *this;
}

void bmsField::getfocus() {
  //cerr << "gotfocus(" << this << ")\n";
  top();
  mark(A_STANDOUT);
  move(0,0);
  sync();
}

bmsScreen::bmsScreen() {
  cur = 0;
}

void bmsScreen::key(int k) {
  switch (k) {
    ChildWin *p;
  case KEY_TAB: case KEY_RIGHT:
    p = (*this)[++cur];
    setfocus(p ? p : (*this)[cur = 0]);
    break;
  case KEY_BTAB: case KEY_LEFT:
    p = (*this)[--cur];
    setfocus(p ? p : (*this)[cur = this->count() - 1]);
    break;
  case KEY_HOME:
    if (cur == 0) return;
    setfocus((*this)[cur = 0]);
    break;
  case KEY_END:
    setfocus((*this)[cur = this->count() - 1]);
    break;
  case KEY_LL:
  default:
    Karet::key(k);
    return;
  }
  sync();
}

void bmsScreen::getfocus() {
  top();
  if (!(*this)[cur]) cur = 0;
  //cerr << "setfocus(" << cur << ")\n";
  setfocus((*this)[cur]);
}

void manScreen::addScrollFld(short len,short plen) {
  bmsField *f = new bmsField(this,len);
  fld.add(f);
  f->setphys(Rect(org,plen));
  org += plen + 1;
}

#include <stdarg.h>

void manScreen::addfld(short len1, int len, ...) {
  addfld(len1);
  va_list p;
  va_start(p,len);
  while (len) {
    addfld(len);
    len = va_arg(p,int);
  }
  va_end(p);
}
