#pragma implementation "Termio.h"
#include "Termio.h"
#include <unistd.h>
#include <stdio.h>

int Termios::open(const char *n,int mode) {
  close();
  fd = ::open(n,mode);
  if (fd != -1)
    save();
  return fd;
}

Termios::Termios(const char *n,int mode) {
  fd = -1;	// open does a close first!
  if (open(n,mode) == -1)
    error(n);
}

void Termios::save() {
  savrc = tcgetattr(fd,&sav);
  if (savrc != -1)
    cur = sav;
}

int Termios::rawmode() const {
  if (cur.c_lflag & ICANON)
    return -1;	// cooked mode
  return cur.c_cc[VMIN];
}

/* not clear how modes should be set for general use */

int Termios::raw(char cnt) {
  if (hwhand() != 0)
    cur.c_iflag = BRKINT;
  else
    cur.c_iflag = IXON | IXOFF | BRKINT;
  cur.c_cflag |= HUPCL | CREAD;
  cur.c_oflag = 0;
#ifndef IEXTEN
#define IEXTEN 0
#endif
  cur.c_lflag = ISIG | IEXTEN;
  cur.c_cc[VMIN] = cnt;
  cur.c_cc[VTIME] = (cnt != 1);
  cur.c_cc[VINTR] = 0;
  /* cur.c_cc[VQUIT] = 0; */
#ifdef VSUSP
  cur.c_cc[VSUSP] = 0;
#endif
  return set();
}

void Termios::echo(bool flag) {
  int cval = cur.c_lflag;
  if (flag)
    cur.c_lflag |= ECHO;
  else
    cur.c_lflag &= ~ECHO;
  if ((cur.c_lflag ^ cval) & ECHO)
    set();
}

int Termios::hwhand(int i) {
  int rc = tcsethwhand(fd,i);
  if (rc == 0) {
    if (i)
      cur.c_iflag &= ~(IXON | IXOFF);
    else 
      cur.c_iflag |= (IXON | IXOFF);
    rc = set();
  }
  return rc;
}

struct {
  unsigned baud, flag;
} btbl[] = {
  { 50, B50 }, { 75, B75 }, { 110, B110 }, { 134, B134 }, { 150, B150 },
  { 200, B200 }, { 300, B300 }, { 600, B600 }, { 1200, B1200 },
  { 1800, B1800 }, { 2400, B2400 }, { 4800, B4800 }, { 9600, B9600 },
  { 19200, B19200 }, { 38400, B38400 },
#ifdef B57600
  { 57600, B57600 }, { 115200, B115200 }, { 230400, B230400 },
#endif
  { 0, 0 }
};

unsigned Termios::baud() const {
  int i;
  for (i = 0; btbl[i].baud; ++i) {
    if ((cur.c_cflag & CBAUD) == btbl[i].flag) break;
  }
  return btbl[i].baud;
}

unsigned Termios::baud(unsigned bd /*,char parity,char bits,char stop */) {
  unsigned old = baud();
  int i;
  for (i = 0; btbl[i].baud; ++i) {
    if (bd == btbl[i].baud) break;
  }
  cur.c_cflag &= ~CBAUD;
  cur.c_cflag |= btbl[i].flag;
  set();
  return old;
}

void Termios::error(const char *n) { perror(n); }

Termios::~Termios() {
  close();
}

void Termios::close() {
  if (fd != -1 && savrc != -1) {
    int savhw = hwhand();
    cur = sav;
    if (savhw > 0) tcsethwhand(fd,0);
    if (setnow() == 0)
      write(2,"Terminal restored\r\n",19);
    else
      perror("tcsetattr");
    if (savhw > 0) tcsethwhand(fd,savhw);
  }
  ::close(fd);
  fd = -1;
}

void Termios::brkint(bool flag) {
  if (flag) {
    cur.c_iflag |= BRKINT;
    cur.c_lflag |= ISIG;
  }
  else {
    cur.c_iflag &= ~BRKINT;
    cur.c_lflag &= ~ISIG;
  }
}

void Termios::open(int f) {
  close();
  fd = f;
  save();
}
